import 'package:flutter/material.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:puma_pay/entities/product/product.dart';
import 'package:puma_pay/entities/purchaseDetail/purchaseDetail.dart';
import 'package:puma_pay/extensions/extProduct.dart';
import 'package:puma_pay/redux/pumaPayActions.dart';
import 'package:puma_pay/redux/pumaPayState.dart';
import 'package:puma_pay/ui/atoms/purchaseDetailTile.dart';
import 'package:puma_pay/ui/templates/purchaseDetailTemplate.dart';
import 'package:puma_ui/helpers/modalBottomSheet.dart';
import 'package:redux/redux.dart';

class PumaPayDebugTemplate<T extends VirtualPumaPayState>
    extends StatelessWidget {
  final String title = "PumaPayDebug";

  PumaPayDebugTemplate();

  @override
  Widget build(BuildContext context) {
    return StoreConnector<T, _ViewModel>(
      converter: (Store<T> store) => _ViewModel.create(store),
      builder: (BuildContext context, _ViewModel viewModel) =>
          _createBody(context, viewModel),
    );
  }

  Widget rowTile({String leading = "", String trailing = ""}) {
    return Padding(
      padding: const EdgeInsets.only(left: 8.0, right: 8.0),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Text(leading),
          Text(trailing),
        ],
      ),
    );
  }

  Widget productRow(
    BuildContext context,
    _ViewModel vm,
    Product product,
  ) {
    return Row(
      children: [
        Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Padding(
              padding: const EdgeInsets.all(8.0),
              child: Icon(Icons.local_mall),
            ),
          ],
        ),
        Expanded(
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Text(
                "${product.title}",
                style: TextStyle(
                  fontWeight: FontWeight.bold,
                ),
              ),
              Text(
                "${product.description}",
                style: Theme.of(context).textTheme.caption,
              ),
              Text(
                "ID: ${product.id}",
                style: Theme.of(context).textTheme.caption,
              ),
              Text(
                "${product.price}",
                style: Theme.of(context).textTheme.caption,
              ),
              Text(
                product.isValidString(),
                style: Theme.of(context).textTheme.bodyText2!.merge(
                      TextStyle(
                        fontWeight: FontWeight.bold,
                        color: product.validColor(),
                      ),
                    ),
              ),
            ],
          ),
        ),
        Column(
          crossAxisAlignment: CrossAxisAlignment.end,
          children: [
            GestureDetector(
              onTap: () {
                vm.store.dispatch(PurchaseProductAction(productID: product.id));
              },
              child: Padding(
                padding: const EdgeInsets.all(8.0),
                child: Chip(
                  label: Text(
                    "Buy Now",
                    style: Theme.of(context).textTheme.caption,
                  ),
                ),
              ),
            ),
          ],
        )
      ],
    );
  }

  Widget getProductPurchases(
    BuildContext context,
    _ViewModel vm,
    Product product,
  ) {
    int purchaseCount = product.purchaseDetails.length;
    List<PurchaseDetail> purchaseDetails = product.purchaseDetails.toList();
    purchaseDetails
        .sort((a, b) => b.transactionDate!.compareTo(a.transactionDate!));

    return Column(
      children: [
        productRow(context, vm, product),
        Row(
          children: [
            Expanded(
              child: Card(
                child: Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Text(
                    "Puchased $purchaseCount times:",
                    style: TextStyle(fontWeight: FontWeight.bold),
                  ),
                ),
              ),
            ),
          ],
        ),
        Column(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            for (var purchaseDetail in purchaseDetails)
              PurchaseDetailTile(
                purchaseDetail: purchaseDetail,
                onTap: () {
                  showPumalModalBottomSheet(
                    context: context,
                    safeArea: true,
                    child: PurchaseDetailTemplate<T>(
                      purchaseDetail: purchaseDetail,
                    ),
                  );
                },
              ),
          ],
        )
      ],
    );
  }

  Widget _createBody(BuildContext context, _ViewModel vm) {
    return Builder(
      builder: (context) {
        return Column(
          children: [
            Container(
              width: MediaQuery.of(context).size.width,
              color: Colors.blueGrey.withAlpha(20),
              child: Padding(
                padding: const EdgeInsets.only(top: 8.0, bottom: 8.0),
                child: Wrap(
                  alignment: WrapAlignment.center,
                  spacing: 4.0,
                  children: [
                    Column(
                      children: [
                        Icon(vm.pumaPayState.storeAvailable
                            ? Icons.check
                            : Icons.motion_photos_off),
                        Text(
                          "Store",
                          style: Theme.of(context).textTheme.caption,
                        ),
                      ],
                    ),
                    Column(
                      children: [
                        if (vm.pumaPayState.productsLoading)
                          Container(
                              width: 20,
                              height: 20,
                              child: CircularProgressIndicator()),
                        if (!vm.pumaPayState.productsLoading)
                          Icon(Icons.motion_photos_off),
                        Text(
                          "Products loading",
                          style: Theme.of(context).textTheme.caption,
                        ),
                      ],
                    ),
                    Column(
                      children: [
                        if (vm.pumaPayState.purchasePending)
                          Container(
                              width: 20,
                              height: 20,
                              child: CircularProgressIndicator()),
                        if (!vm.pumaPayState.purchasePending)
                          Icon(Icons.motion_photos_off),
                        Text(
                          "Purchase Pending",
                          style: Theme.of(context).textTheme.caption,
                        ),
                      ],
                    ),
                    Column(
                      children: [
                        if (vm.pumaPayState.restoringPurchase)
                          Container(
                              width: 20,
                              height: 20,
                              child: CircularProgressIndicator()),
                        if (!vm.pumaPayState.restoringPurchase)
                          Icon(Icons.motion_photos_off),
                        Text(
                          "Restoring Purchase",
                          style: Theme.of(context).textTheme.caption,
                        ),
                      ],
                    ),
                  ],
                ),
              ),
            ),
            Expanded(
              child: ListView(
                children: [
                  Wrap(
                    children: [
                      Column(
                        children: [
                          Padding(
                            padding: const EdgeInsets.all(8.0),
                            child: Text(
                              "Configuration",
                              style: Theme.of(context).textTheme.headline4,
                            ),
                          ),
                          rowTile(
                            leading: "productIDs",
                            trailing: vm.pumaPayState.productIDs.toString(),
                          ),
                          rowTile(
                            leading: "appleAppStorePassword",
                            trailing:
                                "**** ${vm.pumaPayState.appleAppStorePassword.toString().substring(vm.pumaPayState.appleAppStorePassword!.length - 5)}",
                          ),
                          rowTile(
                            leading: "verifyEndpoint",
                            trailing: vm.pumaPayState.verifyEndpoint.toString(),
                          ),
                          Divider(),
                        ],
                      ),
                      Column(
                        children: [
                          Padding(
                            padding: const EdgeInsets.all(8.0),
                            child: Text(
                              "Products Available",
                              style: Theme.of(context).textTheme.headline4,
                            ),
                          ),
                          for (var product in vm.pumaPayState.products)
                            getProductPurchases(context, vm, product),
                        ],
                      ),
                    ],
                  ),
                ],
              ),
            ),
            Container(
              width: MediaQuery.of(context).size.width,
              color: Colors.blue,
              child: SafeArea(
                child: Wrap(
                  alignment: WrapAlignment.center,
                  spacing: 16.0,
                  children: [
                    GestureDetector(
                      onTap: () => vm.store.dispatch(IsStoreAvailableAction()),
                      child: Chip(
                        backgroundColor: Colors.white,
                        label: Text(
                          "Check Availability",
                          style: Theme.of(context).textTheme.caption!.merge(
                                TextStyle(
                                  color: Colors.blue,
                                ),
                              ),
                        ),
                      ),
                    ),
                    GestureDetector(
                      onTap: () => vm.store.dispatch(
                        GetProductsAction(
                          productIDs:
                              vm.store.state.pumaPayState.productIDs.toList(),
                        ),
                      ),
                      child: Chip(
                        backgroundColor: Colors.white,
                        label: Text(
                          "Get Products",
                          style: Theme.of(context).textTheme.caption!.merge(
                                TextStyle(
                                  color: Colors.blue,
                                ),
                              ),
                        ),
                      ),
                    ),
                    GestureDetector(
                      onTap: () => vm.store.dispatch(RestorePurchaseAction()),
                      child: Chip(
                        backgroundColor: Colors.white,
                        label: Text(
                          "Restore",
                          style: Theme.of(context).textTheme.caption!.merge(
                                TextStyle(
                                  color: Colors.blue,
                                ),
                              ),
                        ),
                      ),
                    ),
                  ],
                ),
              ),
            )
          ],
        );
      },
    );
  }
}

class _ViewModel {
  PumaPayState pumaPayState;
  Store store;

  _ViewModel(
    this.pumaPayState,
    this.store,
  );

  factory _ViewModel.create(Store<VirtualPumaPayState> store) {
    return _ViewModel(
      store.state.pumaPayState,
      store,
    );
  }
}
